const parse = require('../parse');
const { normalizeMrz } = require('../normalize-by-ocr');

describe('parse Swiss Driving License', () => {
  it('valid MRZ', () => {
    const MRZ = [
      'AAA001D<<',
      'FACHE305142128097<<800126<<<<<',
      'MARCHAND<<FABIENNE<<<<<<<<<<<<',
    ];
    const result = parse(MRZ);
    expect(result.format).toBe('SWISS_DRIVING_LICENSE');
    expect(result.valid).toStrictEqual(true);
    expect(result.details[0]).toStrictEqual({
      label: 'Document number',
      field: 'documentNumber',
      ranges: [{
        line: 1, start: 5, end: 17, raw: '305142128097',
      }],
      line: 1,
      start: 10,
      end: 22,
      value: '305142128097',
      valid: true,
    });
    expect(result.details[result.details.length - 1]).toStrictEqual({
      label: 'First name',
      field: 'firstName',
      value: 'FABIENNE',
      valid: true,
      ranges: [
        {
          line: 2,
          start: 0,
          end: 30,
          raw: 'MARCHAND<<FABIENNE<<<<<<<<<<<<',
        },
      ],
      line: 2,
      start: 10,
      end: 18,
    });
    expect(result.fields).toStrictEqual({
      documentNumber: '305142128097',
      languageCode: 'D',
      documentCode: 'FA',
      issuingState: 'CHE',
      birthDate: '800126',
      firstName: 'FABIENNE',
      lastName: 'MARCHAND',
    });
  });

  it('valid MRZ 2017', () => {
    const MRZ = [
      'PQE573D<<',
      'FACHE007589314005<<840204<<<<<',
      'STEINMANN<<CHRISTOPH<<<<<<<<<<',
    ];
    const result = parse(MRZ);
    expect(result.format).toBe('SWISS_DRIVING_LICENSE');
    expect(result.valid).toStrictEqual(true);
    expect(result.details.find(({ field }) => field === 'documentNumber').value).toEqual('007589314005');
    expect(result.details.find(({ field }) => field === 'birthDate').value).toEqual('840204');
    expect(result.details.find(({ field }) => field === 'issuingState').value).toEqual('CHE');
    expect(result.details.find(({ field }) => field === 'lastName').value).toEqual('STEINMANN');
    expect(result.details.find(({ field }) => field === 'firstName').value).toEqual('CHRISTOPH');
  });

  it('normalize-mrz', async () => {
    const MRZ = 'EDX243F<<\nFACHE001098108002<<530326<<<<<\nBELLIARD<<MIRIELLE<MARIE<ANTOI';
    const result = await normalizeMrz(MRZ, [{ category: 'Document type', content: 'driver-license', contentType: 'string' }]);
    expect(result.find(({ category }) => category === 'Document number').content).toEqual('001098108002');
    expect(result.find(({ category }) => category === 'Date of birth').content).toEqual('1953-03-26');
    expect(result.find(({ category }) => category === 'Issue country').content).toEqual('CHE');
    expect(result.find(({ category }) => category === 'Last name').content).toEqual('BELLIARD');
    expect(result.find(({ category }) => category === 'First name').content).toEqual('MIRIELLE MARIE ANTOI');
    expect(result.find(({ category }) => category === 'Document type').content).toEqual('driving-licence');
  });
});
