const { normalizeMrz } = require('./normalize-by-ocr');

describe('Parse MRZ', () => {
  it('valid pakistan passport must return data', async () => {
    const mrz = 'P<PAKARIF<<MUHAMMAD<<<<<<<<<<<<<<<<<<<<<<<<<\nAU19874612PAK7606205M11070763740662477467<98';
    const fields = await normalizeMrz(mrz);
    const dict = Object.assign({}, ...fields
      .map(({ category, content }) => ({ [category]: content })));

    expect(dict).toEqual({
      'First name': 'MUHAMMAD',
      'Last name': 'ARIF',
      'Date of birth': '1976-06-20',
      'Date of expiry': '2011-07-07',
      'Document number': 'AU1987461',
      'Document type': 'passport',
      'Issue country': 'PAK',
      Gender: 'male',
      'Nationality code': 'PAK',
      'Optional data': '3740662477467',
      'MRZ full': 'P<PAKARIF<<MUHAMMAD<<<<<<<<<<<<<<<<<<<<<<<<<\nAU19874612PAK7606205M11070763740662477467<98',
    });
    expect(fields.every((f) => f.valid)).toBeTruthy();
  });

  it('valid latvian passport must return data', async () => {
    const pakistanMrz = 'P<LVAMERTENS<<NORMUNDS<<<<<<<<<<<<<<<<<<<<<<\nLV54508288LVA7103232M2606266230371<12527<<90';
    const fields = await normalizeMrz(pakistanMrz);
    const dict = Object.assign({}, ...fields
      .map(({ category, content }) => ({ [category]: content })));

    expect(dict).toEqual({
      'First name': 'NORMUNDS',
      'Last name': 'MERTENS',
      'Date of birth': '1971-03-23',
      'Date of expiry': '2026-06-26',
      'Document number': 'LV5450828',
      'Document type': 'passport',
      'Issue country': 'LVA',
      Gender: 'male',
      'Nationality code': 'LVA',
      'Optional data': '230371 12527',
      'MRZ full': 'P<LVAMERTENS<<NORMUNDS<<<<<<<<<<<<<<<<<<<<<<\nLV54508288LVA7103232M2606266230371<12527<<90',
    });
    expect(fields.every((f) => f.valid)).toBeTruthy();
  });

  it('valid french ID must return data', async () => {
    const mrz = 'IDFRABERTHIER<<<<<<<<<<<<<<<<<<<<<<<\n9409923102854CORINNE<<<<<<<6512068F4';
    const fields = await normalizeMrz(mrz);
    const dict = Object.assign({}, ...fields
      .map(({ category, content }) => ({ [category]: content })));

    expect(dict).toEqual({
      'Date of birth': '1965-12-06',
      'Date of issue': '1994-09-01',
      'Document number': '940992310285',
      'Document type': 'id-card',
      'First name': 'CORINNE',
      Gender: 'female',
      'Issue country': 'FRA',
      'Last name': 'BERTHIER',
      'MRZ full': 'IDFRABERTHIER<<<<<<<<<<<<<<<<<<<<<<<\n9409923102854CORINNE<<<<<<<6512068F4',
    });
    expect(fields.every((f) => f.valid)).toBeTruthy();
  });

  it('valid french ID with authority code must return data', async () => {
    const mrz = 'IDFRATEST<NAME<<<<<<<<<<<<<<<<0CHE02\n1710GVA123451ROBERTA<<<<<<<9112311F2';
    const fields = await normalizeMrz(mrz);
    const dict = Object.assign({}, ...fields
      .map(({ category, content }) => ({ [category]: content })));

    expect(dict).toEqual({
      'Date of birth': '1991-12-31',
      'Date of issue': '2017-10-01',
      'Document number': '1710GVA12345',
      'Document type': 'id-card',
      'First name': 'ROBERTA',
      Gender: 'female',
      'Issue country': 'FRA',
      'Last name': 'TEST NAME',
      'MRZ full': 'IDFRATEST<NAME<<<<<<<<<<<<<<<<0CHE02\n1710GVA123451ROBERTA<<<<<<<9112311F2',
    });
    expect(fields.every((f) => f.valid)).toBeTruthy();
  });

  it('invalid mrz does not return data', async () => {
    const pakistanMrz = 'P<LVAMERTENS<<NORMUNDSS<<<<<<<<<<<<<<<<<<<<<<\nLV54508288LVA7103232M2606266230371<12527<<90';
    const errorMessage = 'unrecognized document format. First line of input must have 30 (TD1), 36 (TD2 or French National Id), 44 (TD3) or 9 (Swiss Driving License) characters';
    await expect(normalizeMrz(pakistanMrz)).rejects.toThrow(errorMessage);
  });

  it('not valid check sum for doc number', async () => {
    const pakistanMrz = 'P<LVAMERTENS<<NORMUNDS<<<<<<<<<<<<<<<<<<<<<<\nLV34508288LVA7103232M2606266230371<12527<<90';
    const fields = await normalizeMrz(pakistanMrz);
    const firstName = fields.find((x) => x.category === 'First name');
    const docNumber = fields.find((x) => x.category === 'Document number');
    expect(firstName.content).toBe('NORMUNDS');
    expect(firstName.valid).toBe(true);
    expect(docNumber.content).toBe('LV3450828');
    expect(docNumber.valid).toBe(false);
  });

  it('not valid check sum for date of birth', async () => {
    const pakistanMrz = 'P<LVAMERTENS<<NOMMUNDS<<<<<<<<<<<<<<<<<<<<<<\nLV54508288LVA7103222M2606266230371<12527<<90';
    const fields = await normalizeMrz(pakistanMrz);
    const date = fields.find((x) => x.category === 'Date of birth');
    expect(date.content).toBe('1971-03-22');
    expect(date.valid).toBe(false);
  });

  it('not valid check sum for date of expiry', async () => {
    const pakistanMrz = 'P<LVAMERTENS<<NOMMUNDS<<<<<<<<<<<<<<<<<<<<<<\nLV54508288LVA7103232M2606256230371<12527<<90';
    const fields = await normalizeMrz(pakistanMrz);
    const date = fields.find((x) => x.category === 'Date of expiry');
    expect(date.content).toBe('2026-06-25');
    expect(date.valid).toBe(false);
  });

  it('in valid summary', async () => {
    const lines = 'I<PER40259864<4<<<<<<<<<<<<<<<\n7809198M2510169ER<<<<<<<<<<<8<\nGONZALES<<WALTER<PAUL<<<<<<<<<';
    const fields = await normalizeMrz(lines);
    const full = fields.find((x) => x.category === 'MRZ full');
    expect(full.valid).toBe(false);
  });

  it('valid RP card', async () => {
    const lines = 'RPESTBB00308371<<<<<<<<<<<<<<<\n8511294M2204053RUS<<<<<<<<<<<2\nPOPOV<<VLADIMIR<<<<<<<<<<<<<<<';
    const fields = await normalizeMrz(lines);
    const full = fields.find((x) => x.category === 'MRZ full');
    expect(full.valid).toBe(true);
  });

  it('valid mrz with personal number as optional data', async () => {
    const lines = 'P<UTOERIKSSON<<ANNA<MARIA<<<<<<<<<<<<<<<<<<<\nL898902C<3UTO6908061F9406236ZE184226B<<<<<14';
    const fields = await normalizeMrz(lines);
    const optionalData = fields.find((x) => x.category === 'Optional data');
    expect(optionalData.valid).toBe(true);
  });

  it('valid deutch mrz', async () => {
    const lines = 'P<D<<SCHNELL<<GERRIT<REINHARD<<<<<<<<<<<<<<<\nC72XY19P10D<<9206090M2810227<<<<<<<<<<<<<<<8';
    const fields = await normalizeMrz(lines);
    const country = fields.find((x) => x.category === 'Issue country');
    expect(country.valid).toBe(true);
    expect(country.content).toBe('DEU');
  });

  it('valid france resident-permit', async () => {
    const mrz = 'IRFRAHYOHDHE7D2<1303197792<<<<\n8103066M2301147JPN<<<<<<<<<<<4\nFUJITA<<SEIJI<<<<<<<<<<<<<<<<<';
    const fields = await normalizeMrz(mrz);
    const type = fields.find((x) => x.category === 'Document type');
    expect(type.valid).toBe(true);
    expect(type.content).toBe('residence-permit');
  });

  it('v1', async () => {
    const mrz = 'PNRUSERMOLAEV<<ANATOLIQ<VLADIMIROVI3<<<<<<<<\n4611841180RUS9606151M<<<<<<<6160622500148<36';
    const fields = await normalizeMrz(mrz);
    const mrz1 = fields.find((x) => x.category === 'MRZ full');
    expect(mrz1.valid).toBe(true);
  });

  it('valid belgian resident-permit', async () => {
    const mrz = 'IDBELLE<MEUNIER<<JENNIFER<ANNE<<<<<<\nP9992034<3BEL6002017F0008017<<<<<<<4';
    const fields = await normalizeMrz(mrz);
    const type = fields.find((x) => x.category === 'Document type');
    expect(type.valid).toBe(true);
    expect(type.content).toBe('residence-permit');
  });

  it('valid belgian id-card', async () => {
    const mrz = 'IDBEL592394609<6238<<<<<<<<<<<\n9408296M2606200BEL940829327570\nDUQUENNE<<MICKAEL<MONIQUE<T<<<';
    const fields = await normalizeMrz(mrz);
    const type = fields.find((x) => x.category === 'Document type');
    expect(type.valid).toBe(true);
    expect(type.content).toBe('id-card');
  });

  it('valid id-card check (starts with ID)', async () => {
    const mrz = 'IDFRANABIS<<<<<<<<<<<<<<<<<<<<97A006\n120597A003379YANN<<SAMUEL<<9701229M0';
    const fields = await normalizeMrz(mrz);
    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const type = fields.find((x) => x.category === 'Document type');
    expect(type.content).toBe('id-card');
  });

  it('valid Azerbaijan id-card (starts with IA)', async () => {
    const mrz = 'IAAZEAA012345650123456<<<<<<<<\n8012311M4012313AZE<<<<<<<<<<<3\nJOHN<<DOE<<<<<<<<<<<<<<<<<<<<<';
    const fields = await normalizeMrz(mrz);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    expect(fields.find((x) => x.category === 'Document type')).toEqual({
      valid: true,
      content: 'id-card',
      category: 'Document type',
      contentType: 'string',
    });
  });

  it('valid Chile id-card', async () => {
    const mrz = 'IECHL0123456784S01<<<<<<<<<<<<\n8012311M4012313ARG12345678<9<0\nJUAN<<PABLO<<<<<<<<<<<<<<<<<<<';
    const fields = await normalizeMrz(mrz);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    expect(fields.find((x) => x.category === 'Document type')).toEqual({
      valid: true,
      content: 'id-card',
      category: 'Document type',
      contentType: 'string',
    });
  });

  it('valid mrz with symbols "O" instead of zero', async () => {
    const mrz = 'IECHLO123456784S01<<<<<<<<<<<<\n8O12311M4O12313ARG12345678<9<0\nJUAN<<PABLO<<<<<<<<<<<<<<<<<<<';
    const fields = await normalizeMrz(mrz);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const docNumber = fields.find((x) => x.category === 'Document number');
    expect(docNumber).toEqual({
      valid: true,
      content: '012345678',
      category: 'Document number',
      contentType: 'string',
    });
    const expDate = fields.find((x) => x.category === 'Date of expiry');
    expect(expDate).toEqual({
      valid: true,
      content: '2040-12-31',
      category: 'Date of expiry',
      contentType: 'date',
      extractedFormat: 'YYYY-MM-DD',
    });
  });

  it('valid mrz with HUN corner case', async () => {
    const mrz = 'I<HUN506373AH<7<<<<<<<<<<<<<<<\n4811206F8001014HUN<<<<<<<<<<<2\nURBAN<<SANDORNE<<<<<<<<<<<<<<<';
    const fields = await normalizeMrz(mrz);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const expDate = fields.find((x) => x.category === 'Date of expiry');
    expect(expDate).toEqual({
      valid: true,
      content: '2080-01-01',
      category: 'Date of expiry',
      contentType: 'date',
      extractedFormat: 'YYYY-MM-DD',
    });
  });

  it('valid lva mrz', async () => {
    const mrz = 'INLVANA00516087081078<12719<<<\n7810082M2108210XXX<<<<<<<<<<<6\nJARKIVSKIS<<PIKALIJS<<<<<<<<<<';
    const fields = await normalizeMrz(mrz);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const docType = fields.find((x) => x.category === 'Document type');
    expect(docType).toEqual({
      valid: true,
      content: 'id-card',
      category: 'Document type',
      contentType: 'string',

    });
  });

  it('valid ROU mrz', async () => {
    const mrz = 'IDROUIONAS<<ANDREI<<<<<<<<<<<<<<<<<<\nXB619594<4ROU5105025M790502311913234';
    const fields = await normalizeMrz(mrz);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const expirationDate = fields.find((x) => x.category === 'Date of expiry');
    expect(expirationDate).toEqual({
      category: 'Date of expiry',
      content: '2079-05-02',
      contentType: 'date',
      valid: true,
      extractedFormat: 'YYYY-MM-DD',
    });
  });

  it('HRV mrz', async () => {
    const mrz = 'IOHRV113565199437791507390<<<<\n5212213M9912315HRV<<<<<<<<<<<9\nSTUK<<ARMIN<<<<<<<<<<<<<<<<<<<';
    const fields = await normalizeMrz(mrz);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const expirationDate = fields.find((x) => x.category === 'Date of expiry');
    expect(expirationDate).toEqual({
      category: 'Date of expiry',
      content: '9999-12-31',
      contentType: 'date',
      valid: true,
      extractedFormat: 'YYYY-MM-DD',
    });
  });

  it('ESP mrz', async () => {
    const mrz = 'IDESPBJZ132805834241920A<<<<<<\n4809186M9901018ESP<<<<<<<<<<<9\nGONZALEZ<CASAS<<VICTOR<FRANCIS';
    const fields = await normalizeMrz(mrz);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const expirationDate = fields.find((x) => x.category === 'Date of expiry');
    expect(expirationDate).toEqual({
      category: 'Date of expiry',
      content: '9999-01-01',
      contentType: 'date',
      valid: true,
      extractedFormat: 'YYYY-MM-DD',
    });
  });

  it('HRV driver licence mrz', async () => {
    const mrz = 'D11058590748065570952280426<<6';
    const ocr = [
      {
        category: 'Issue country',
        content: 'HRV',
      },
    ];
    const fields = await normalizeMrz(mrz, ocr);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const expirationDate = fields.find((x) => x.category === 'Date of expiry');
    expect(expirationDate).toEqual({
      category: 'Date of expiry',
      content: '2026-04-28',
      contentType: 'date',
      valid: true,
      extractedFormat: 'YYYY-MM-DD',
    });
  });

  it('IRL driver licence mrz after 2013-01-19', async () => {
    const mrz = 'D<NOLAN<<<<<<<IRL<0000JAFP9002';
    const ocr = [
      {
        category: 'Issue country',
        content: 'IRL',
      },
      {
        category: 'Date of issue',
        content: '2014-08-19',
      },
    ];
    const fields = await normalizeMrz(mrz, ocr);

    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const expirationDate = fields.find((x) => x.category === 'Document number');
    expect(expirationDate).toEqual({
      category: 'Document number',
      content: '0000JAFP9',
      contentType: 'string',
      error: undefined,
      valid: true,
    });
  });

  it('IRL driver licence mrz before 2013-01-19', async () => {
    const mrz = 'D<NOLAN<<<<<<<IRL<0000JAFP9002';
    const ocr = [
      {
        category: 'Issue country',
        content: 'IRL',
      },
      {
        category: 'Date of issue',
        content: '2012-08-19',
      },
    ];
    const fields = await normalizeMrz(mrz, ocr);
    for (const field of fields) {
      expect(field).toMatchObject({ valid: true });
    }
    const documentNumber = fields.find((x) => x.category === 'Document number');
    expect(documentNumber).toEqual({
      category: 'Document number',
      content: '0000JAFP90',
      contentType: 'string',
      error: undefined,
      valid: true,
    });
  });

  it('MEX id-card', async () => {
    const mrz = 'CGMEX2000002226<<<<<<<<<<<<<<<\n7006115M1911149MEX<<<<<<<<<<<6\nORIOL<TORRE<<LUIS<JOSE<<<<<<<<';
    const fields = await normalizeMrz(mrz, []);
    const documentType = fields.find((x) => x.category === 'Document type');
    expect(documentType).toEqual({
      category: 'Document type',
      content: 'id-card',
      contentType: 'string',
      valid: true,
    });
  });

  it('MEX voter-card', async () => {
    const mrz = 'IDMEX2000230999<<1236063715000\n7902022H2912316MEX<05<<19936<1\nSOTO<VALENCIA<<JUAN<CARLOS<<<<';
    const fields = await normalizeMrz(mrz, []);
    const documentType = fields.find((x) => x.category === 'Document type');
    expect(documentType).toEqual({
      category: 'Document type',
      content: 'voter-card',
      contentType: 'string',
      valid: true,
    });
  });

  it('document number', async () => {
    const mrz = 'D<ESTEV257748<39603095711<<<<8';
    const ocr = [
    ];
    const fields = await normalizeMrz(mrz, ocr);
    const documentType = fields.find((x) => x.category === 'Document number');
    expect(documentType).toEqual({
      category: 'Document number',
      content: 'EV257748',
      contentType: 'string',
      valid: true,
    });
  });

  it('invalid doc number', async () => {
    const mrz = 'IDD<<L0F5476X47<<<<<<<<<<<<<<<\n7908220<2607274D<<<<<<<<<<<<<8\nHAMANN<<KATHARINA<<<<<<<<<<<<<';
    const ocr = [
      {
        category: 'Document number',
        content: 'L0F5476X9',
        contentType: 'string',
      },
    ];
    const fields = await normalizeMrz(mrz, ocr);
    const documentType = fields.find((x) => x.category === 'Document number');
    expect(documentType).toEqual({
      category: 'Document number',
      content: 'L0F5476X4',
      contentType: 'string',
      valid: false,
    });
  });

  it('invalid doc code', async () => {
    const mrz = '123<<L0F5476X47<<<<<<<<<<<<<<<\n7908220<2607274D<<<<<<<<<<<<<8\nHAMANN<<KATHARINA<<<<<<<<<<<<<';
    const ocr = [
      {
        category: 'Document number',
        content: 'L0F5476X9',
        contentType: 'string',
      },
    ];
    const fields = await normalizeMrz(mrz, ocr);
    const documentType = fields.find((x) => x.category === 'Document type');
    expect(documentType).toEqual({
      category: 'Document type',
      content: 'unknown',
      contentType: 'string',
      valid: true,
    });
  });
});
